--***************************************************************
--* File         : TB_WBVER.vhd                                 *
--* Created      : time-stamp: <08/08/26 01:54:33 kawasan>      *
--* Modified     : Time-stamp: <08/08/26 02:26:32 kawasan>      *
--*								*
--* Test-Bench                                                  *
--*								*
--* Copyright 2008 Osamu Kawashima. all rights reserved		*
--***************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use std.textio.all;
use ieee.std_logic_textio.all;

entity TB_WBVER is end;

architecture behavior of TB_WBVER is
    
	component WBVER
	generic (
		-- Version-Code = ver AB01 (rel 01)
		VERSION_CODE0   : std_logic_vector := x"41";	-- A:Device_Name-High
		VERSION_CODE1   : std_logic_vector := x"42";	-- B:Device_Name-Low
		VERSION_CODE2   : std_logic_vector := x"30";	-- 0:Version-High
		VERSION_CODE3   : std_logic_vector := x"31";	-- 1:Version-Low
		RELEASE_CODE0	: std_logic_vector := x"30";	-- 0:Release-High
		RELEASE_CODE1	: std_logic_vector := x"31"	-- 1:Release-Low
	);
	port (
		-- WISHBONE SLAVE interface:
		WB_CLK_I        : in    std_logic;
		WB_RST_I        : in    std_logic;
		WB_ADR_I        : in    std_logic_vector(7 downto 0);
		WB_DAT_I        : in    std_logic_vector(7 downto 0);
		WB_WE_I         : in    std_logic;
		WB_STB_I        : in    std_logic;
		WB_ACK_O        : out   std_logic;
		WB_DAT_O        : out   std_logic_vector(7 downto 0)
	);
	end component;

	constant ZERO32         : std_logic_vector(31 downto 0) := (others=>'0');
	constant ONE32          : std_logic_vector(31 downto 0) := (others=>'1');

	constant WT1            : Time := 10 ns;
	constant WT_DELAY       : Time := 1 ns;
	constant WT_IDLE        : Time := 100 ns;
	constant CLK_CYCLE      : Time := 21 ns;	-- about 48MHz

	signal WB_CLK_I         : std_logic := '0';
	signal WB_RST_I         : std_logic := '1';
	signal WB_ADR_I         : std_logic_vector(7 downto 0) := (others=>'0');
	signal WB_DAT_I         : std_logic_vector(7 downto 0) := (others=>'0');
	signal WB_WE_I          : std_logic := '0';
	signal WB_STB_I         : std_logic := '0';
	signal WB_ACK_O         : std_logic := '0';
	signal WB_DAT_O         : std_logic_vector(7 downto 0) := (others=>'0');

	signal t_add            : std_logic_vector(WB_ADR_I'range) := (others=>'0');
	signal rd_data          : std_logic_vector(WB_DAT_O'range) := (others=>'0');
	signal rd_dat           : std_logic_vector(WB_DAT_O'range) := (others=>'0');

	-- WB Read (8bit)
	procedure rd_ub(
		ADD_I		: in    std_logic_vector(7 downto 0);
		DAT_I		: in    std_logic_vector(7 downto 0);
	signal	ADD_O		: out   std_logic_vector(7 downto 0);
	signal	DAT_O		: out   std_logic_vector(7 downto 0);
	signal	STB		: out   std_logic;
	signal	WE		: out   std_logic
	) is
	begin
		wait until WB_CLK_I'event and WB_CLK_I='1';
		wait for WT_DELAY;
		ADD_O	<= ADD_I;
		STB	<= '1';
		WE	<= '0';
		wait until WB_CLK_I'event and WB_CLK_I='1';
		wait for WT_DELAY;
		DAT_O	<= DAT_I;
		wait for WT_DELAY;
		ADD_O	<= (others=>'0');
		STB	<= '0';
		WE	<= '0';
	end rd_ub;

begin
    
	uWBVER : WBVER
	generic map (
		-- Version-Code
		VERSION_CODE0   => x"55",
		VERSION_CODE1   => x"AA",
		VERSION_CODE2   => x"11",
		VERSION_CODE3   => x"22",
		RELEASE_CODE0	=> x"44",
		RELEASE_CODE1	=> x"88"
	)
	port map (
		-- WISHBONE SLAVE interface:
		WB_CLK_I        => WB_CLK_I,
		WB_RST_I        => WB_RST_I,
		WB_ADR_I        => WB_ADR_I,
		WB_DAT_I        => WB_DAT_I,
		WB_WE_I         => WB_WE_I,
		WB_STB_I        => WB_STB_I,
		WB_ACK_O        => WB_ACK_O,
		WB_DAT_O        => WB_DAT_O
	);

	process(WB_RST_I,WB_CLK_I)
	begin
		if WB_RST_I='1' then
			rd_dat <= (others=>'0');
		elsif WB_CLK_I'event and WB_CLK_I='1' then
			if WB_STB_I='1' and WB_WE_I='0' then
				rd_dat <= WB_DAT_O;
			else
				rd_dat <= rd_dat;
			end if;
		end if;
	end process;

	process begin
		WB_CLK_I <= '0';
		wait for CLK_CYCLE/2;
		WB_CLK_I <= '1';
		wait for CLK_CYCLE/2;
	end process;

	process
		variable ln         : line;
	begin
		-- Reset
		wait for WT_IDLE+WT_DELAY;
		WB_RST_I <= '0';
		write(ln, string'("--- Release Reset. ---"));
		writeline(output, ln);
		wait for WT_IDLE;

		-- dump-read
		T_ADD <= x"00";
		for iCnt in 1 to 8 loop
			wait for WT_IDLE/2;
			rd_ub(T_ADD, WB_DAT_O, WB_ADR_I, rd_data, WB_STB_I, WB_WE_I);
			hwrite(ln, rd_dat);
			writeline(output, ln);
			T_ADD <= T_ADD + 1;
		end loop;

		-- End
		write(ln, string'("--- End of Test-Bench. ---"));
		writeline(output, ln);
		wait;
	end process;

end behavior;
